@@ -1,20 +1,20 @@
-"""quixote
+"""Quixote
 
-A highly Pythonic web application framework.
+A small and flexible Python web application framework.
 """
 
-__version__ = '2.6'
+__version__ = '2.8'
 
 # These are frequently needed by Quixote applications.
 from quixote.publish import \
      get_publisher, get_request, get_response, get_path, redirect, \
      get_session, get_session_manager, get_user, get_field, get_cookie, \
-     get_wsgi_app, cleanup
+     get_wsgi_app, cleanup, get_param
 
 
 # This is the default charset used by the HTTPRequest, HTTPResponse,
 # DefaultLogger, and sendmail components.
-DEFAULT_CHARSET = 'iso-8859-1'
+DEFAULT_CHARSET = 'utf-8'
 
 def enable_ptl():
     """
