@@ -10,7 +10,7 @@ import os
 import tempfile
 import urllib
 import rfc822
-from cStringIO import StringIO
+from StringIO import StringIO
 
 import quixote
 from quixote.http_response import HTTPResponse
@@ -155,7 +155,7 @@ class HTTPRequest:
         # sets this environment variable to "0" for non-SSL requests
         # (most web servers -- well, Apache at least -- simply don't set
         # it in that case).
-        if (environ.get('HTTPS', 'off').lower() == 'on' or
+        if (environ.get('HTTPS', 'off').lower() in ('on', 'yes', '1') or
             environ.get('SERVER_PORT_SECURE', '0') != '0'):
             self.scheme = "https"
         else:
@@ -188,13 +188,34 @@ class HTTPRequest:
             length = int(length)
         except ValueError:
             raise RequestError('invalid content-length header')
+        read_body = length > 0
         ctype = self.environ.get("CONTENT_TYPE")
         if ctype:
             ctype, ctype_params = parse_header(ctype)
             if ctype == 'application/x-www-form-urlencoded':
                 self._process_urlencoded(length, ctype_params)
+                read_body = False
             elif ctype == 'multipart/form-data':
                 self._process_multipart(length, ctype_params)
+                read_body = False
+        if read_body:
+            # We must consume entire request body as some clients and
+            # middleware expect that.  We cannot rely on the application to
+            # read it completely (e.g. if there is some PublishError raised).
+            if length < 20000:
+                fp = StringIO()
+            else:
+                fp = tempfile.TemporaryFile("w+b")
+            remaining = length
+            while remaining > 0:
+                s = self.stdin.read(min(remaining, 10000))
+                if not s:
+                    raise RequestError('unexpected end of request body')
+                fp.write(s)
+                remaining -= len(s)
+            fp.seek(0)
+            self._stdin = self.stdin
+            self.stdin = fp
 
     def _process_urlencoded(self, length, params):
         query = self.stdin.read(length)
@@ -364,6 +385,17 @@ class HTTPRequest:
         """
         return self.environ.get('QUERY_STRING', '')
 
+    def get_path_query(self):
+        """() -> string
+
+        Return the path and the query string (if any).
+        """
+        path = self.get_path()
+        query = self.get_query()
+        if query:
+            path += '?' + query
+        return path
+
     def get_url(self, n=0):
         """get_url(n : int = 0) -> string
 
@@ -397,7 +429,7 @@ class HTTPRequest:
         found_encodings = self._parse_pref_header(accept_encoding)
         if found_encodings:
             for encoding in encodings:
-                if found_encodings.has_key(encoding):
+                if encoding in found_encodings:
                     return encoding
         return None
 
@@ -736,12 +768,12 @@ class LineInput:
 class MIMEInput:
     """
     Split a MIME input stream into parts.  Note that this class does not
-    handle headers, transfer encoding, etc.  
+    handle headers, transfer encoding, etc.
     """
 
     def __init__(self, fp, boundary, length):
         self.lineinput = LineInput(fp, length)
-        self.pat = re.compile(r'--%s(--)?[ \t]*\r\n' % re.escape(boundary))
+        self.pat = re.compile(r'--%s(--)?' % re.escape(boundary))
         self.done = False
 
     def moreparts(self):
