@@ -4,7 +4,6 @@ Provides the HTTPResponse class.
 """
 
 import time
-from sets import Set
 try:
     import zlib
 except ImportError:
@@ -70,7 +69,8 @@ _GZIP_HEADER = ("\037\213" # magic
                 "\002"
                 "\377")
 
-_GZIP_EXCLUDE = Set(["application/pdf",
+# content that is already compressed, don't bother trying
+_GZIP_EXCLUDE = set(["application/pdf",
                      "application/zip",
                      "audio/mpeg",
                      "image/gif",
@@ -125,10 +125,10 @@ class HTTPResponse:
         future requests.  The cookie value is stored as the "value"
         attribute.  The other attributes are as specified by RFC 2109.
       cache : int | None
-        the number of seconds the response may be cached.  The default is 0,
-        meaning don't cache at all.  This variable is used to set the HTTP
-        expires header.  If set to None then the expires header will not be
-        added.
+        the number of seconds the response may be cached.  The default
+        is 0, meaning don't cache at all.  This variable is used to set
+        the HTTP expires and cache-control headers.  If set to None then
+        no headers will be added.
       javascript_code : { string : string }
         a collection of snippets of JavaScript code to be included in
         the response.  The collection is built by calling add_javascript(),
@@ -139,7 +139,7 @@ class HTTPResponse:
     DEFAULT_CONTENT_TYPE = 'text/html'
     DEFAULT_CHARSET = None # defaults to quixote.DEFAULT_CHARSET
 
-    
+
     def __init__(self, status=200, body=None, content_type=None, charset=None):
         """
         Creates a new HTTP response.
@@ -160,6 +160,7 @@ class HTTPResponse:
         self.cache = 0
         self.buffered = True
         self.javascript_code = None
+        self._allow_chunked = False
 
     def set_content_type(self, content_type, charset=None):
         """(content_type : string, charset : string = None)
@@ -198,7 +199,7 @@ class HTTPResponse:
 
         self.status_code = status
         if reason is None:
-            if status_reasons.has_key(status):
+            if status in status_reasons:
                 reason = status_reasons[status]
             else:
                 # Eg. for generic 4xx failures, use the reason
@@ -251,32 +252,45 @@ class HTTPResponse:
         """(body: str) -> str
         """
         n = len(body)
-        co = zlib.compressobj(6, zlib.DEFLATED, -zlib.MAX_WBITS,
-                              zlib.DEF_MEM_LEVEL, 0)
-        crc = zlib.crc32(body)
-        chunks = [_GZIP_HEADER,
-                  co.compress(body),
-                  co.flush(),
-                  struct.pack("<LL", _LOWU32(crc), _LOWU32(n))]
-        compressed_body = "".join(chunks)
+        compressed_body = ''.join(self._generate_compressed([body]))
         ratio = float(n) / len(compressed_body)
         #print "gzip original size %d, ratio %.1f" % (n, ratio)
         if ratio > 1.0:
+            # only compress if we save space
             self.set_header("Content-Encoding", "gzip")
             return compressed_body
         else:
             return body
 
+    def _generate_compressed(self, body):
+        co = zlib.compressobj(6, zlib.DEFLATED, -zlib.MAX_WBITS,
+                                  zlib.DEF_MEM_LEVEL, 0)
+        crc = zlib.crc32('') & 0xffffffffL
+        n = 0
+        yield _GZIP_HEADER
+        for chunk in body:
+            if not isinstance(chunk, str):
+                chunk = self._encode_chunk(stringify(chunk))
+            crc = zlib.crc32(chunk, crc) & 0xffffffffL
+            n += len(chunk)
+            yield co.compress(chunk)
+        crc = struct.pack("<LL", _LOWU32(crc), _LOWU32(n))
+        yield co.flush() + crc
+
     def set_body(self, body, compress=False):
         """(body : any, compress : bool = False)
 
         Sets the response body equal to the argument 'body'.  If 'compress'
-        is true then the body may be compressed using 'gzip'.
+        is true then the body may be compressed.
         """
         if not isinstance(body, Stream):
             body = self._encode_chunk(stringify(body))
             if compress and self.content_type not in _GZIP_EXCLUDE:
                 body = self._compress_body(body)
+        else:
+            if compress and self.content_type not in _GZIP_EXCLUDE:
+                self.set_header("Content-Encoding", "gzip")
+                body = Stream(self._generate_compressed(body))
         self.body = body
 
     def expire_cookie(self, name, **attrs):
@@ -296,15 +310,13 @@ class HTTPResponse:
         self.set_cookie(name, "deleted", **dict)
 
     def set_cookie(self, name, value, **attrs):
-        """set_cookie(name : string, value : string, **attrs)
-
-        Set an HTTP cookie on the browser.
+        """Set an HTTP cookie on the browser.
 
         The response will include an HTTP header that sets a cookie on
         cookie-enabled browsers with a key "name" and value "value".
         Cookie attributes such as "expires" and "domains" may be
         supplied as keyword arguments; see RFC 2109 for a full list.
-        (For the "secure" attribute, use any true value.)
+        (For the "secure" and httponly attributes, use any true value.)
 
         This overrides any previous value for this cookie.  Any
         previously-set attributes for the cookie are preserved, unless
@@ -312,7 +324,7 @@ class HTTPResponse:
         call.
         """
         cookies = self.cookies
-        if cookies.has_key(name):
+        if name in cookies:
             cookie = cookies[name]
         else:
             cookie = cookies[name] = {}
@@ -332,7 +344,7 @@ class HTTPResponse:
         """
         if self.javascript_code is None:
             self.javascript_code = {code_id: code}
-        elif not self.javascript_code.has_key(code_id):
+        elif code_id not in self.javascript_code:
             self.javascript_code[code_id] = code
 
     def redirect(self, location, permanent=False):
@@ -368,6 +380,11 @@ class HTTPResponse:
         else:
             return len(self.body)
 
+    def enable_transfer_chunked(self):
+        """Allow response to be sent as "Transfer-Encoding: chunked"
+        """
+        self._allow_chunked = True
+
     def _gen_cookie_headers(self):
         """_gen_cookie_headers() -> [string]
 
@@ -389,6 +406,8 @@ class HTTPResponse:
                     chunks.append('%s=%s' % (name, val))
                 elif name == 'secure' and val:
                     chunks.append("secure")
+                elif name == 'httponly' and val:
+                    chunks.append("httponly")
             cookie_headers.append(("Set-Cookie", '; '.join(chunks)))
         return cookie_headers
 
@@ -397,30 +416,45 @@ class HTTPResponse:
 
         Generate a list of headers to be returned as part of the response.
         """
-        headers = []
-
-        for name, value in self.headers.items():
-            headers.append((name.title(), value))
-
-        # All the "Set-Cookie" headers.
-        if self.cookies:
-            headers.extend(self._gen_cookie_headers())
-
         # Date header
         now = time.time()
         if "date" not in self.headers:
-            headers.append(("Date", formatdate(now)))
+            self.headers['date'] =  formatdate(now)
 
         # Cache directives
-        if self.cache is None:
-            pass # don't mess with the expires header
-        elif "expires" not in self.headers:
+        if self.cache is None or self.headers.has_key("expires"):
+            pass # don't mess with the expires or cache control header
+        else:
+            # We add both an Expires header and a Cache-Control header
+            # with a max-age directive.  The max-age directive takes
+            # priority when both Expires and max-age are present (even
+            # if Expires is more restrictive, RFC 2616 section 14.9.3).
             if self.cache > 0:
                 expire_date = formatdate(now + self.cache)
+                cache_control = "max-age=%d" % self.cache
             else:
-                expire_date = "-1" # allowed by HTTP spec and may work better
-                                   # with some clients
-            headers.append(("Expires", expire_date))
+                # This is the default case and makes sense for a
+                # dynamically generated response that can change on each
+                # request.
+                #
+                # Using the current date is not a good idea since clocks
+                # might not be synchronized. Any invalid date is treated
+                # as in the past but Microsoft recommends "-1" for
+                # Internet Explorer so that's what we use.
+                expire_date = "-1"
+                # The Expires header is sufficient for HTTP 1.0 but
+                # for HTTP 1.1 we must add a must-revalidate directive.
+                # Clients and proxies are allowed to ignore Expires in
+                # certain cases and use stale pages (RFC 2616 sections
+                # 13.1.5 and 14.9.4).
+                cache_control = "max-age=0, must-revalidate"
+            if ("expires" not in self.headers and
+                    "cache-control" not in self.headers):
+                # If either of these headers are set then don't add
+                # any of them. We assume the programmer knows what he
+                # is doing in that case.
+                self.headers['expires'] = expire_date
+                self.headers['cache-control'] = cache_control
 
         # Content-type
         if "content-type" not in self.headers:
@@ -428,27 +462,62 @@ class HTTPResponse:
                 value = '%s; charset=%s' % (self.content_type, self.charset)
             else:
                 value = '%s' % self.content_type
-            headers.append(('Content-Type', value))
+            self.headers['content-type'] = value
 
         # Content-Length
         if "content-length" not in self.headers:
             length = self.get_content_length()
             if length is not None:
-                headers.append(('Content-Length', str(length)))
+                self.headers['content-length'] = str(length)
+            elif self._allow_chunked:
+                # No content-length and chunked encoding possible,
+                # use it.
+                self.headers['transfer-encoding'] = 'chunked'
+            else:
+                self.headers['connection'] = 'close'
 
+        headers = []
+        for name, value in self.headers.items():
+            headers.append((name.title(), value))
+        if self.cookies:
+            # All the "Set-Cookie" headers.
+            headers.extend(self._gen_cookie_headers())
         return headers
 
-    def generate_body_chunks(self):
+    def _generate_encoded_body(self):
         """Return a sequence of body chunks, encoded using 'charset'.
         """
         if self.body is None:
             pass
         elif isinstance(self.body, Stream):
             for chunk in self.body:
-                yield self._encode_chunk(chunk)
+                if not isinstance(chunk, str):
+                    chunk = self._encode_chunk(chunk)
+                yield chunk
         else:
             yield self.body # already encoded
 
+    def _generate_transfer_chunked(self, stream):
+        """Convert a sequence of encoded body data into the format
+        # expected by "Transfer-Encoding: chunked".
+        """
+        # Each chunk is as follows:
+        #    <length of data hex><CRLF>
+        #    <data><CRLF>
+        #    <CRLF>
+        # The stream is terminated by a zero length chunk.
+        for chunk in stream:
+            if chunk:
+                yield ''.join(['%x\r\n' % len(chunk), chunk, '\r\n'])
+        yield '0\r\n\r\n'
+
+    def generate_body_chunks(self):
+        stream = self._generate_encoded_body()
+        if self.headers.get('transfer-encoding') == 'chunked':
+            return self._generate_transfer_chunked(stream)
+        else:
+            return stream
+
     def write(self, output, include_status=True, include_body=True):
         """(output:file, include_status:bool=True, include_body:bool=True)
 
