@@ -138,6 +138,9 @@ class SessionManager:
         """
         return self.sessions.get(session_id, default)
 
+    def __iter__(self):
+        return self.sessions.itervalues()
+
     def __getitem__(self, session_id):
         """(session_id : string) -> Session
 
@@ -152,11 +155,13 @@ class SessionManager:
         Return true if a session identified by 'session_id' exists in
         the session manager.
         """
-        return self.sessions.has_key(session_id)
+        return session_id in self.sessions
 
-    # has_session() is a synonym for has_key() -- if you override
-    # has_key(), be sure to repeat this alias!
-    has_session = has_key
+    def __contains__(self, session_id):
+        return self.has_key(session_id)
+
+    def has_session(self, session_id):
+        return self.has_key(session_id)
 
     def __setitem__(self, session_id, session):
         """(session_id : string, session : Session)
@@ -308,6 +313,11 @@ class SessionManager:
             if not path.endswith("/"):
                 path += "/"
         domain = config.session_cookie_domain
+        attrs = attrs.copy()
+        if config.session_cookie_secure:
+            attrs['secure'] = 1
+        if config.session_cookie_httponly:
+            attrs['httponly'] = 1
         get_response().set_cookie(name, value, domain=domain,
                                   path=path, **attrs)
         return name
